/*******************************************************************************
 * (c) Copyright 2008 Actel Corporation
 * 
 *
 * this code runs on both the M1AFS-Embedded-Kit 
 * and the M1AFS-ADV-DEV-KIT
 */

#include "Core51S_test_IO.h"
#include "SC_8051s_reg51.h"
#include "stdio.h"

/******************************************************************************/
/*							function prototypes								  */
/******************************************************************************/

unsigned char key_pressed( void );
void init_sio(void);
void UART_send(const char* tx_buffer, unsigned char tx_size);
void scan_channels(void);
extern void write_uart (char c);
void init_ADC (char varef, char mode, char clk_div);
void init_quad (char quad_byte_addr, char quad_data);
unsigned int read_ADC_ch(char sample, char channel);

/******************************************************************************/
/*							CoreAI definitions								  */
/******************************************************************************/

#define ACM_busy	0x1C
#define ACM_reset	0x01
#define ACM_rd_busy	0x08
#define ACM_rd_start 0x02
#define AQ0_B0_addr 0x01			// AV0 control
#define AQ0_B1_addr	0x02			// AC0 control
#define AQ1_B0_addr 0x05			// AV1 control
#define AQ1_B1_addr	0x06			// AC1 control
#define AQ2_B0_addr 0x09			// AV2 control
#define AQ2_B3_addr	0x0C			// AT2 control
#define AQ3_B0_addr 0x0D			// AV3 control
#define AQ4_B0_addr 0x11			// AV4 control
#define AQ4_B1_addr 0x12			// AC4 control
#define AQ8_B0_addr 0x21			// AV8 control
#define Pos_2V		0x83			//
#define Pos_4V		0x82			//
#define Pos_8V		0x81			//
#define Pos_curr	0x10			// current mode
#define Pos_temp	0x10			// temperature mode
#define ADC_CAL		0x80			// cal busy
#define ADC_start	0x20
#define ADC_BUSY	0x20			// converion busy
#define ADC_SAMPLE	0x40			// sample busy
#define ADC_reset	0x40
#define int_vref	0x00
#define mode_12_bit	0x0D			// no pwrdwn, no cal, 12-bit
#define ADC_clk		0x00			// sysclk /4
#define ADC_AV0_ch	0x01			// ch = 1 dec
#define	ADC_AC0_ch	0x02			// ch = 2 dec
#define ADC_AV1_ch	0x04			// ch = 4 dec
#define ADC_AC1_ch	0x05			// ch = 5 dec
#define ADC_AT2_ch	0x09			// ch = 9 dec
#define ADC_AC4_ch	0x0E			// ch = 14 dec
#define ADC_AV8_ch	0x19			// ch = 25 dec
#define ADC_sample	80				// sample and hold = n+2 ADC clks
#define curr_sample	0xFF			// sample and hold = n+2 ADC clks
#define	cur_stb_0	0x01			// current strobe AC0
#define	cur_stb_1	0x02			// current strobe AC1
#define	temp_stb_2	0x04			// current strobe AT2


/******************************************************************************/
/*							text string definitions							  */
/******************************************************************************/

const unsigned char AV0_msg[] = "    AV0     :  ";
const unsigned char AC0_msg[] = "    AC0     :  ";
const unsigned char AV1_msg[] = "    AV1     :  ";
const unsigned char AC1_msg[] = "    AC1     :  ";
const unsigned char AT2_msg[] = "    AT2     :  ";
const unsigned char AC4_msg[] = "    AC4     :  ";
  
const unsigned char sep[] = "==========================================================================";
const unsigned char mesg[] = "Press any key to display current analog inputs values.";

const unsigned char bar[] = {~0, ~1, ~3, ~7, ~15};

/******************************************************************************/
/*							other definitions								  */
/******************************************************************************/

#define baud 					78						// 9600 baud @ 12MHz
#define uart_ctrl_2 			0x01
#define bar_step				819						// step size for bar graph

/******************************************************************************/
/*							start of main code								  */
/******************************************************************************/

int main( void )
{

   	init_ADC (int_vref, mode_12_bit, ADC_clk);			// init the ADC mode, clocks, ref
 	init_sio();											// init the UART
    printf("\n%s\n%s\n", sep,mesg); 					// display start message
 
 	ADCcon3 = 0;
 	ADCcon4 = 0;
     
    while( 1 )
    {
        if (key_pressed())
        	{
            	scan_channels();
        	}
    }
}

/******************************************************************************/
/*							key pressed function							  */
/******************************************************************************/

unsigned char key_pressed( void )
{
    unsigned char ret_val = 0;
	unsigned char c;
	
    if (SStat & 0x02)
    	{
        	ret_val = 1;
        	c =  RxData;					// read the char to clear status
    	}
    return ret_val;
}

/******************************************************************************/
/*							scan ADC channels function						  */
/******************************************************************************/

void scan_channels(void)
{
	
	unsigned int i;
	
	printf("\n%s\n", sep);								// display_update_sep
	
	init_quad (AQ0_B0_addr, Pos_4V);					// AV0 set for voltage
	i = read_ADC_ch(ADC_sample, ADC_AV0_ch);			// read AV0
	printf("%s%d mV\n",AV0_msg, i);						// print AV0
		
	ADCcon3 = 0;										// clear current strobe for AC0
	init_quad (AQ0_B0_addr, Pos_curr);					// AV0 set for current
	init_quad (AQ0_B1_addr, Pos_curr);					// AC0 set for current	
	for (i=0; i < 100; i++);							// delay for current measurement
	ADCcon3 = cur_stb_0;								// set current strobe for AC0
	for (i=0; i < 10; i++);								// delay for current measurement
	i = read_ADC_ch(curr_sample, ADC_AC0_ch);			// read AC0
	printf("%s%d mA\n",AC0_msg, i/16, i);				// print AC0
	
	init_quad (AQ1_B0_addr, Pos_2V);					// AV1 set for voltage
	i = read_ADC_ch(ADC_sample, ADC_AV1_ch);			// read AV1
	printf("%s%d mV\n",AV1_msg, i/2);					// print AV1
	ADCcon3 = cur_stb_1;								// set current strobe for AC1
	init_quad (AQ1_B0_addr, Pos_curr);					// AV1 set for current
	init_quad (AQ1_B1_addr, Pos_curr);					// AC1 set for current
	for (i=0; i < 10; i++);								// delay for current measurement
	i = read_ADC_ch(curr_sample, ADC_AC1_ch);			// read AC1
	printf("%s%d mA\n",AC1_msg, i/16, i);				// print AC1
	
	init_quad (AQ2_B3_addr, Pos_temp);					// AT2 set for temperature
	for (i=0; i < 100; i++);							// delay for temperature measurement
	ADCcon4 = temp_stb_2;								// set temperature strobe for AT2
	for (i=0; i < 20; i++);								// delay for temperature measurement
	i = read_ADC_ch(ADC_sample, ADC_AT2_ch);			// read AT2
	printf("%s%d C\n",AT2_msg,((i/4) - 278));			// print AT2 - adj for 273 K and 5 deg pos offset
	
	init_quad (AQ4_B1_addr, Pos_4V);					// AC4
	i = read_ADC_ch(ADC_sample, ADC_AC4_ch);			// read AC4
	printf("%s%d mV\n",AC4_msg, i);						// print AC4
	dataOut = bar[ i/bar_step];							// drive bargraph
	
}	
  
/******************************************************************************/
/*							init ADC function								  */
/******************************************************************************/

void init_ADC (char varef, char mode, char clk_div) 
 	{
 		unsigned char i, j;

 		do
			{
				i = ADCstat;							// read APB
				j = XRB1;
			}
		while (j & (ADC_BUSY | ADC_CAL | ADC_SAMPLE));	// wait until ADC is not busy
 					
 		XWB1 = clk_div;
 		ADCcon1 = ((varef << 4 ) & 0x10) |(mode & 0x0F);				
	}
 	
/******************************************************************************/
/*							init analog quad function						  */
/******************************************************************************/

void init_quad (char quad_byte_addr, char quad_data)

	{
		while (ACMcon & ACM_busy);				// wait until ACM is not busy
		ACMaddr = quad_byte_addr;				// set addr for quad control byte
		ACMdata = quad_data;					// set the quad control byte value
		while (ACMcon & ACM_busy);				// wait until ACM is not busy					
	}

/******************************************************************************/
/*					convert & read ADC channel function					      */
/******************************************************************************/

unsigned int read_ADC_ch(char sample, char channel)

	{
		unsigned char i, j;
		unsigned int k;
				
 		do
 			{
 				i = ADCstat;								// read APB
 				j = XRB1;
 			}
		while (j & (ADC_BUSY | ADC_CAL | ADC_SAMPLE));		// wait until ADC is not busy				
		XWB1 = (channel & 0x1F);
		ADCcon2 = sample;									// write channel number & sample clks
		XWB1 = (channel & 0x1F)| ADC_start;
		ADCcon2 = sample;									// start conversion
		
		do
 			{
 				i =  ADCstat;
 				j = XRB1;									// read APB				 
 			}	
		while (j & ADC_BUSY);								// wait until ADC is not busy
		k = ((j & 0x0F)* 0x100) + i;
		XWB1 = 0;
		ADCcon3 = 0;										// turn off current strobes
		XWB1 = 0;
		ADCcon4 = 0;										// turn off temperature strobes
		return (k);		
	}	



/******************************************************************************/
/*							display seperator  function						  */
/******************************************************************************/

void display_input_sep( void )
{
    unsigned char inp_sep[] = "     |      ";
    
    printf("%s\n", inp_sep);
}

/******************************************************************************/
/*							init UART function								  */
/******************************************************************************/

void init_sio(void)

	{
		Scon1 = baud;							// 19.2Kbaud	
		Scon2 = uart_ctrl_2;					// 8-bit chars							
	}
