/*******************************************************************************
 * (c) Copyright 2008 Actel Corporation
 * 
 */
 
 // target is M1AFS-DEV-KIT-SCS-SA

#include "Core51S_test_IO.h"
#include "SC_8051s_reg51.h"
#include "stdio.h"

/******************************************************************************/
/*							function prototypes								  */
/******************************************************************************/

unsigned char key_pressed( void );
void init_sio(void);
void UART_send(const char* tx_buffer, unsigned char tx_size);
void scan_channels(void);
extern void write_uart (char c);
void init_ADC (char varef, char mode, char clk_div);
void init_quad (char quad_byte_addr, char quad_data);
unsigned int read_ADC_ch(char sample, char channel);

/******************************************************************************/
/*							CoreAI definitions								  */
/******************************************************************************/


#define ACM_busy	0x1C
#define ACM_reset	0x01
#define ACM_rd_busy	0x08
#define ACM_rd_start 0x02

#define AQ6_B0_addr 0x19			// AV6 control (4*6) +1 
#define AQ6_B1_addr	0x1A			// AC6 control
#define AQ7_B0_addr 0x1D			// AV7 control
#define AQ7_B1_addr	0x1E			// AC7 control
#define AQ8_B0_addr 0x21			// AV8 control
#define AQ8_B3_addr	0x24			// AT8 control
#define AQ9_B0_addr 0x25			// AV9 control
#define AQ9_B1_addr 0x26			// AC9 control
#define AQ9_B3_addr	0x28			// AT9 control
#define Pos_2V		0x83			//
#define Pos_4V		0x82			//
#define Pos_8V		0x81			//
//#define Pos_curr	0x10			// current mode
#define Pos_curr	0x92			// current mode
//#define Pos_temp	0x10			// temperature mode
#define Pos_temp	0x92			// temperature mode
#define ADC_CAL		0x80			// cal busy
#define ADC_start	0x20
#define ADC_BUSY	0x20			// converion busy
#define ADC_SAMPLE	0x40			// sample busy
#define ADC_reset	0x40
#define int_vref	0x00
#define mode_12_bit	0x0D			// no pwrdwn, no cal, 12-bit
#define ADC_clk		0x00			// sysclk /4
#define ADC_AV6_ch	0x13			// ch = 19 dec
#define ADC_AC6_ch	0x14			// ch = 20 dec
#define ADC_AV7_ch	0x16			// ch = 22 dec
#define ADC_AC7_ch	0x17			// ch = 23 dec
#define ADC_AV8_ch	0x19			// ch = 25 dec
#define ADC_AT8_ch	0x1B			// ch = 27 dec
#define ADC_AV9_ch	0x1C			// ch = 28 dec
#define ADC_AC9_ch	0x1D			// ch = 29 dec
#define ADC_AT9_ch	0x1E			// ch = 30 dec
#define ADC_sample	28				// sample and hold = n+2 ADC clks
#define curr_sample	28				// sample and hold = n+2 ADC clks

#define	cur_stb_6	0x40			// current strobe AC6, ADCon3 (low)
#define	cur_stb_7	0x80			// current strobe AC7, ADCon3 (low)
#define	cur_stb_9	0x02			// current strobe AC9, ADCon3 (hi)
#define	temp_stb_8	0x01			// current strobe AT8, ADCon4 (high)
#define	temp_stb_9	0x02			// current strobe AT9, ADCon4 (high)

#define	curr_low	60				// current strobe low delay 60 = 5us
#define	curr_high	40				// current strobe high delay
#define	temp_low	60				// temp strobe low delay
#define	temp_high	60				// temp strobe high delay

/******************************************************************************/
/*							text string definitions							  */
/******************************************************************************/

const unsigned char AV6_msg[] = "    AV6     :  ";
const unsigned char AC6_msg[] = "    AC6     :  ";
const unsigned char AV7_msg[] = "    AV7     :  ";
const unsigned char AC7_msg[] = "    AC7     :  ";
const unsigned char AV8_msg[] = "    AV8     :  ";
const unsigned char AT8_msg[] = "    AT8     :  ";
const unsigned char AV9_msg[] = "    AV9     :  ";
const unsigned char AC9_msg[] = "    AC9     :  ";
const unsigned char AT9_msg[] = "    AT9     :  "; 
 
const unsigned char sep[] = "==========================================================================";
const unsigned char mesg[] = "Press any key to display current analog inputs values.";

const unsigned char bar[] = {~0, ~1, ~3, ~7, ~15};

/******************************************************************************/
/*							other definitions								  */
/******************************************************************************/

#define baud 					78						// 9600 baud @ 12MHz
//#define baud 					156						// 9600 baud @ 24MHz
#define uart_ctrl_2 			0x01
//#define bar_step				819						// step size for bar graph

#define bar_step				440					// step size for bar graph

/******************************************************************************/
/*							start of main code								  */
/******************************************************************************/

int main( void )
{
   	init_ADC (int_vref, mode_12_bit, ADC_clk);			// init the ADC mode, clocks, ref 	
 	init_sio();											// init the UART
    printf("\n%s\n%s\n", sep,mesg); 					// display start message
 
 	ADCcon3 = 0;
 	ADCcon4 = 0;
     
    while( 1 )
    {
        if (key_pressed())
        	{
            	scan_channels();
        	}
    }
}

/******************************************************************************/
/*							key pressed function							  */
/******************************************************************************/

unsigned char key_pressed( void )
{
    unsigned char ret_val = 0;
	unsigned char c;
	
    if (SStat & 0x02)
    	{
        	ret_val = 1;
        	c =  RxData;					// read the char to clear status
    	}
    return ret_val;
}

/******************************************************************************/
/*							scan ADC channels function						  */
/******************************************************************************/

void scan_channels(void)
{
	
	unsigned int i;
	
	printf("\n%s\n", sep);								// display_update_sep
	init_quad (AQ6_B0_addr, Pos_4V);					// AV6
	i = read_ADC_ch(ADC_sample, ADC_AV6_ch);			// read AV6
	printf("%s%d mV\n",AV6_msg, i);						// print AV6	
	ADCcon3 = 0;										// clear current strobe for AC7:0
	init_quad (AQ6_B0_addr, Pos_curr);					// AV6	prep'ed for current on AC6
	init_quad (AQ6_B1_addr, Pos_curr);					// AC6
	for (i=0; i < curr_low; i++);						// delay for current measurement
	ADCcon3 = cur_stb_6;								// set current strobe for AC6
	for (i=0; i < curr_high; i++);						// delay for current measurement
	i = read_ADC_ch(curr_sample, ADC_AC6_ch);			// read AC6
	ADCcon3 = 0;										// clear current strobe for AC7:0
	printf("%s%d mA\n",AC6_msg,(i*10)/16);				// print AC6
	
	init_quad (AQ7_B0_addr, Pos_2V);					// AV7
	i = read_ADC_ch(ADC_sample, ADC_AV7_ch);			// read AV7
	printf("%s%d mV\n",AV7_msg, i/2);					// print AV7
	ADCcon3 = 0;										// clear current strobe for AC7:0
	init_quad (AQ7_B0_addr, Pos_curr);					// AV7 prep'ed for current on AC7
	init_quad (AQ7_B1_addr, Pos_curr);					// AC7
	for (i=0; i < curr_low; i++);						// delay for current measurement	
	ADCcon3 = cur_stb_7;								// set current strobe for AC7
	for (i=0; i < curr_high; i++);						// delay for current measurement
	i = read_ADC_ch(curr_sample, ADC_AC7_ch);			// read AC7
	ADCcon3 = 0;										// clear current strobe for AC7:0
	printf("%s%d mA\n",AC7_msg,(i*10)/16);				// print AC7
	
	init_quad (AQ8_B0_addr, Pos_8V);					// AV8	
	i = read_ADC_ch(ADC_sample, ADC_AV8_ch);			// read AV8
	printf("%s%d mV\n",AV8_msg, i * 2);					// print AV8
	dataOut = (~(bar[ i/bar_step])) & 0x0F;				// drive bargraph for the pot on AV8
	init_quad (AQ8_B3_addr, Pos_temp);					// AT8	
	XWB1 = 0;
	ADCcon4 = 0;										// clear temperature strobe for AT9:8
	for (i=0; i < temp_low; i++);						// delay for temperature measurement
	XWB1 = temp_stb_8;		
	ADCcon4 = 0;										// set temperature strobe for AT8
	for (i=0; i < temp_high; i++);						// delay for temperature measurement	
	i = read_ADC_ch(ADC_sample, ADC_AT8_ch);			// read AT8
	XWB1 = 0;
	ADCcon4 = 0;										// clear temperature strobe for AT9:8
	printf("%s%d C\n",AT8_msg,((i/4) - 278));			// print AT8 - adj for 273 K and 5 deg pos offset
	
	init_quad (AQ9_B0_addr, Pos_4V);					// AV9
	i = read_ADC_ch(ADC_sample, ADC_AV9_ch);			// read AV9
	printf("%s%d mV\n",AV9_msg, i);						// print AV9
	XWB1 = 0;
	ADCcon3 = 0;										// clear current strobe for AC9:8
	init_quad (AQ9_B0_addr, Pos_curr);					// AV9 prep'ed for current on AC9
	init_quad (AQ9_B1_addr, Pos_curr);					// AC9
	for (i=0; i < curr_low; i++);						// delay for current measurement
	XWB1 = cur_stb_9;
	ADCcon3 = 0;										// set current strobe for AC9
	for (i=0; i < curr_high; i++);						// delay for current measurement
	i = read_ADC_ch(curr_sample, ADC_AC9_ch);			// read AC9
	XWB1 = 0;
	ADCcon3 = 0;										// clear current strobe for AC9:8
	printf("%s%d mA\n",AC9_msg,(i*10)/16);				// print AC9
	
	init_quad (AQ9_B3_addr, Pos_temp);					// AT9
	XWB1 = 0;
	ADCcon4 = 0;										// clear temperature strobe for AT9:8
	for (i=0; i < temp_low; i++);						// delay for temperature measurement
	XWB1 = temp_stb_9;		
	ADCcon4 = 0;										// set temperature strobe for AT9
	for (i=0; i < temp_high; i++);						// delay for temperature measurement				
	i = read_ADC_ch(ADC_sample, ADC_AT9_ch);			// read AT9
	XWB1 = 0;
	ADCcon4 = 0;										// clear temperature strobe for AT9:8
	printf("%s%d C\n",AT9_msg,((i/4) - 278));			// print AT9 - adj for 273 K and 5 deg pos offset	
}	
 
/******************************************************************************/
/*							init ADC function								  */
/******************************************************************************/

void init_ADC (char varef, char mode, char clk_div) 
 	{
 		unsigned char i, j;

 		do
			{
				i = ADCstat;							// read APB
				j = XRB1;
			}
		while (j & (ADC_BUSY | ADC_CAL | ADC_SAMPLE));	// wait until ADC is not busy
 					
 		XWB1 = clk_div;
 		ADCcon1 = ((varef << 4 ) & 0x10) |(mode & 0x0F);				
	}
 	
/******************************************************************************/
/*							init analog quad function						  */
/******************************************************************************/

void init_quad (char quad_byte_addr, char quad_data)

	{
		while (ACMcon & ACM_busy);				// wait until ACM is not busy
		ACMaddr = quad_byte_addr;				// set addr for quad control byte
		ACMdata = quad_data;					// set the quad control byte value
		while (ACMcon & ACM_busy);				// wait until ACM is not busy					
	}

/******************************************************************************/
/*					convert & read ADC channel function					      */
/******************************************************************************/

unsigned int read_ADC_ch(char sample, char channel)

	{
		unsigned char i, j;
		unsigned int k;
				
 		do
 			{
 				i = ADCstat;								// read APB
 				j = XRB1;
 			}
		while (j & (ADC_BUSY | ADC_CAL | ADC_SAMPLE));		// wait until ADC is not busy				
		XWB1 = (channel & 0x1F);
		ADCcon2 = sample;									// write channel number & sample clks
		XWB1 = (channel & 0x1F)| ADC_start;
		ADCcon2 = sample;									// start conversion
		
		do
 			{
 				i =  ADCstat;
 				j = XRB1;									// read APB				 
 			}	
		while (j & ADC_BUSY);								// wait until ADC is not busy

		XWB1 = 0;
		ADCcon3 = 0;										// turn off current strobes
		ADCcon4 = 0;										// turn off temperature strobes
		k = ((j & 0x0F)* 0x100) + i;

		return (k);		
	}	



/******************************************************************************/
/*							display seperator  function						  */
/******************************************************************************/

void display_input_sep( void )
{
    unsigned char inp_sep[] = "     |      ";
    
    printf("%s\n", inp_sep);
}

/******************************************************************************/
/*							init UART function								  */
/******************************************************************************/

void init_sio(void)

	{
		Scon1 = baud;							// 19.2Kbaud	
		Scon2 = uart_ctrl_2;					// 8-bit chars							
	}
